@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 #
 # Install Btrfsd build dependencies
 #
@@ -31,3 +31,7 @@ eatmydata apt-get install -yq --no-install-recommends \
     gtk-doc-tools \
     libmount-dev \
     libjson-glib-dev
+
+if apt-cache show systemd-dev > /dev/null 2>&1; then
+    eatmydata apt-get install -yq systemd-dev
+fi
